WheelRotations = {};

function WheelRotations:prerequisitesPresent(specializations)
	return true;
end;

function WheelRotations:load(xmlFile)
	self.wheelRotations = {};
	local i = 0;

	while true do
		local baseName = string.format("vehicle.wheelRotations.wheelRotation(%d)", i);

		if not hasXMLProperty(xmlFile, baseName) then
			break;
		end

		local node = Utils.indexToObject(self.components, getXMLString(xmlFile, baseName .. "#index"));
		local wheelIndex = getXMLInt(xmlFile, baseName .. "#wheelIndex");

		if node ~= nil and self.wheels[wheelIndex + 1] ~= nil then
			table.insert(self.wheelRotations, {
				node = node,
				wheel = self.wheels[wheelIndex + 1]
			});
		end;

		i = i + 1;
	end;

	for i, wheel in ipairs(self.wheels) do
		local wheelnamei = string.format("vehicle.wheels.wheel(%d)", wheel.xmlIndex);

		if hasXMLProperty(xmlFile, wheelnamei .. "#rotSpeedLimit") then
			wheel.rotSpeedLimit = getXMLInt(xmlFile, wheelnamei .. "#rotSpeedLimit");
			wheel.rotSpeedDefault = wheel.rotSpeed;
			wheel.rotSpeedNegDefault = wheel.rotSpeedNeg;
			wheel.currentRotSpeedAlpha = 1;
		end;
	end;
end;

function WheelRotations:delete()
end;

function WheelRotations:mouseEvent(posX, posY, isDown, isUp, button)
end;

function WheelRotations:keyEvent(unicode, sym, modifier, isDown)
end;

function WheelRotations:update(dt)
	if self.getIsActive(self) and self.firstTimeRun then
		for i, wheelRotation in pairs(self.wheelRotations) do
			_, y, _ = getRotation(wheelRotation.wheel.repr);

			setRotation(wheelRotation.node, 0, y, 0);
		end;
	end;
end;

function WheelRotations:updateTick(dt)
	if self.getIsActive(self) then
		for k, wheel in pairs(self.wheels) do
			if wheel.rotSpeedLimit ~= nil then
				local dir = -1;

				if self.getLastSpeed(self) <= wheel.rotSpeedLimit then
					dir = 1;
				end;

				wheel.currentRotSpeedAlpha = Utils.clamp(wheel.currentRotSpeedAlpha + dir*dt/1000, 0, 1);
				wheel.rotSpeed = wheel.rotSpeedDefault*wheel.currentRotSpeedAlpha;
				wheel.rotSpeedNeg = wheel.rotSpeedNegDefault*wheel.currentRotSpeedAlpha;
			end;
		end;
	end;
end;
	
function WheelRotations:draw()
end;


